//game.cpp
#include "globals.h"

GameClass::GameClass()
{
	vid=sf::VideoMode(800, unsigned int((800*9)/16), 32);
	app.Create(vid, "Grig Saves Time");
	app.SetFramerateLimit(60);
	mapIndex=3;
	init();
}

void GameClass::init()
{
	quit=false;
	hud.SetCenter(float(vid.Width/2), float(vid.Height/2));
	hud.SetHalfSize(float(vid.Width/2), float(vid.Height/2));
	theImage.LoadFromFile("images/spritesheet.png");
	theImage.SetSmooth(false);
	player.loadFrame(theImage, 0);
	player.setLeft(false);
	map[0].initMap("maps/map02.txt");
	map[0].startPoint=sf::Vector2f(64, 256-32);
	map[1].initMap("maps/map01.txt");
	map[1].startPoint=sf::Vector2f(64, 160);
	map[2].initMap("maps/map03.txt");
	map[2].startPoint=sf::Vector2f(64, 160);
	map[3].initMap("maps/map04.txt");
	map[3].startPoint=sf::Vector2f(64, 96);
	map[4].initMap("maps/map05.txt");
	map[4].startPoint=sf::Vector2f(14*32, 224);
	background.LoadFromFile("images/title.png");
	background.SetSmooth(false);
	bkSprite.SetImage(background);
	storyBoard=0;
	feyTricks.LoadFromFile("sounds/fey-tricks.wav");
	clockSaver.LoadFromFile("sounds/saved-the-clock.wav");

	for(int j=0; j<3; j++)
	{
		for(int i=0; i<10; i++)
		{
			mob[i].pos=map[j].mobPoint[i];
			mob[i].mobStyle=map[j].mobStyle[i];
			mob[i].vel.x=(2.0f+(1-rand()%3))/3.0f;
			if(mapIndex==0)
				mob[i].isDead=true;
		}
	}
	float height=32*9;
	float vidwidth=32*16;
	float width=(height*16.0f/9.0f);
	float difference=(width-vidwidth)/2.0f;
	camport=sf::FloatRect(-difference, 0.0f, vidwidth+(difference*2), height);
	camera.SetFromRect(camport);

	curState=gs_TITLE;
}

void GameClass::theLoop()
{
	char cl[16];
	bool isKeyAttack=false;
	while(!quit)
	{
		frameClock.Reset();
		//input handling
		if(app.GetEvent(theEvent))
		{
			if(theEvent.Type == sf::Event::Closed) quit=true;
			if(theEvent.Type == sf::Event::Resized)
			{
				float height=32*9;
				float vidwidth=32*16;
				float width=(height*16.0f/9.0f);
				float difference=(width-vidwidth)/2.0f;
				camport=sf::FloatRect(-difference, 0.0f, vidwidth+difference, height);
				camera.SetFromRect(camport);

			}
			else
			{
				if(isState(gs_TITLE))
				{
					background.LoadFromFile("images/title.png");
					background.SetSmooth(false);
					bkSprite.SetImage(background);
					mapIndex=0;
					if(theEvent.Type==sf::Event::KeyPressed)
					{
						curState=gs_MENU;
					}
				}
				if(isState(gs_WIN))
				{
					if(theEvent.Type==sf::Event::KeyPressed)
					{
						if(theEvent.Key.Code == sf::Key::Escape || theEvent.Key.Code == sf::Key::Return || theEvent.Key.Code == sf::Key::Space)
							curState=gs_TITLE;
					}
				}
				else if(isState(gs_MENU))
				{
					if(theEvent.Type==sf::Event::KeyPressed)
					{
						transitionClock.Reset();
						if(theEvent.Key.Code==sf::Key::Escape)
							curState=gs_LEVEL_TRANSITION;
						else
						{
							storyBoard++;
							if(storyBoard>3)
							{					
								themeSong.SetBuffer(feyTricks);
								themeSong.SetLoop(true);
								themeSong.Play();

								curState=gs_LEVEL_TRANSITION;
							}
							else curState=gs_MENU;
						}
					}
				}
				else if(isState(gs_PLAY))
				{
					if(theEvent.Type==sf::Event::KeyPressed)
					{
						if(theEvent.Key.Code==sf::Key::R)
							curState=gs_RESTARTING;
						else curState=gs_PLAY;
					}
				}
			}
		}
		//key input
		if(isState(gs_PLAY) && !player.isDead)
		{
			if(app.GetInput().IsKeyDown(sf::Key::Right))
			{
				player.vel.x+=1.1f;
				player.setLeft(false);
			}
			else if(app.GetInput().IsKeyDown(sf::Key::Left))
			{
				player.vel.x-=1.1f;
				player.setLeft(true);
			}
			if(app.GetInput().IsKeyDown(sf::Key::Space))
			{
				if(player.jumptime==0)
				{
					if(!player.isJumping)
					{
						player.isJumping=true;
						player.vel.y -= 15;
					}
					else
					{
						if(player.jumptime<player.maxjump && !player.isFalling)
						{
							player.vel.y -= 15;
						}
					}
				}
			}
			if(app.GetInput().IsKeyDown(sf::Key::C))
			{
				if(!isKeyAttack && player.attackClock.GetElapsedTime()>0.45f && !player.isAttacking) 
				{
					player.isAttacking=true;
					player.attackClock.Reset();
				}
				isKeyAttack=true;
			}
			else
			{
				isKeyAttack=false;
				player.isAttacking=false;
			}

		}

		// update handling
		switch(curState)
		{
		case gs_MENU:
			hudText.SetPosition(float(vid.Width/2), float(vid.Height/2));
			hudText.SetText(" ");
			break;
		case gs_PLAY:
			hudText.SetPosition(14, 14);
			sprintf_s(cl, sizeof(cl), "Seconds: %i", player.collected);
			hudText.SetText(cl);
			break;
		case gs_LEVEL_TRANSITION:
			hudText.SetPosition(float(vid.Width/2)-(12*7), float(vid.Height/2));
			hudText.SetText("Loading Level");
			break;
		case gs_WIN:
			hudText.SetPosition(float(vid.Width/2), float(vid.Height/2));
			hudText.SetText(" ");
			storyBoard=4;
			break;
		case gs_DIE:
			hudText.SetPosition(float(vid.Width/2)-(12*8), float(vid.Height/2));
			hudText.SetText("You're the Loser");
			break;
		case gs_RESTARTING:
			hudText.SetPosition(float(vid.Width/2)-(12*6), float(vid.Height/2));
			hudText.SetText("Restarting");
			break;
		default:
			hudText.SetPosition(float(vid.Width/2)-(12*3), float(vid.Height/2));
			hudText.SetText(" ");
		}
		if(isState(gs_PLAY))
		{
			player.update(map[mapIndex], mob, seconds);
			if(mapIndex>0) 
			{
				for(int i=0; i<10; i++)
				{
					mob[i].update(map[mapIndex]);
					if(mob[i].isDead && !mob[i].isDropped && !seconds[i].isActive)
					{
						seconds[i].isActive=true;
						seconds[i].isFalling=true;
						seconds[i].loadFrame(theImage, 3);
						seconds[i].pos=mob[i].pos;
						seconds[i].vel.y=-10.0f;
						seconds[i].vel.x=float(1-rand()%3);
						mob[i].isDropped=true;
					}
					seconds[i].update(map[mapIndex]);
				}
			}
			map[mapIndex].update();
			transitionClock.Reset();
		}
		else if(isState(gs_LEVEL_TRANSITION) || isState(gs_RESTARTING))
		{
			player.init();
			player.pos=map[mapIndex].startPoint;
			player.vel.x=0.0f;
			player.update(map[mapIndex], mob, seconds);
			for(int i=0; i<10; i++)
			{
				mob[i].pos=map[mapIndex].mobPoint[i];
				mob[i].mobStyle=map[mapIndex].mobStyle[i];
				mob[i].vel.x=2.0f+(1-rand()%3);
				mob[i].isDead=false;
				mob[i].isDying=false;
				mob[i].isDropped=false;
				seconds[i].isActive=false;
				seconds[i].vel=sf::Vector2f(0,0);
				player.collected=0;
			}

			if(mapIndex>0)
			{
				for(int i=0; i<10; i++)
				{
					mob[i].update(map[mapIndex]);
					mob[i].isDead=false;
					mob[i].isDying=false;
				}
			}
			map[mapIndex].update();
			if(transitionClock.GetElapsedTime()>1.25f)
			{
				curState=gs_PLAY;
			}

		}
		// render handling
		app.Clear(sf::Color(173, 220, 218));
		app.SetView(camera);
		//camera stuff
		if(isState(gs_PLAY) || isState(gs_LEVEL_TRANSITION))
		{
			map[mapIndex].render(app);
			player.render(app, theImage);
			if(mapIndex>0) 
			{
				for(int i=0; i<10; i++)
				{
					mob[i].render(app, theImage);
					seconds[i].render(app, theImage);
				}
			}
			else player.collected=10;
		}
		if(isState(gs_TITLE))
		{
			bkSprite.SetPosition(0,0);
			app.Draw(bkSprite);
		}
		if(isState(gs_MENU) || isState(gs_WIN))
		{
			switch(storyBoard)
			{
				case 0: background.LoadFromFile("images/story1.png"); break;
				case 1: background.LoadFromFile("images/story2.png"); break;
				case 2: background.LoadFromFile("images/story3.png"); break;
				case 3: background.LoadFromFile("images/story4.png"); break;
				case 4: default: background.LoadFromFile("images/story5.png"); break;
			}
			bkSprite.SetImage(background);
			bkSprite.SetPosition(0,0);
			app.Draw(bkSprite);
		}
		app.SetView(hud);
		//hud stuff
		app.Draw(hudText);

		while(frameClock.GetElapsedTime()<0.019f);
		app.Display();
		if(player.isDead && player.deathClock.GetElapsedTime()>2.0f) {curState=gs_RESTARTING; player.isDead=false; player.pos=map[mapIndex].startPoint;}
		if(player.isAscended) {
			curState=gs_LEVEL_TRANSITION;
			player.init();
			mapIndex++;
			player.pos=map[mapIndex].startPoint;
			for(int i=0; i<10; i++)
			{
				mob[i].pos=map[mapIndex].mobPoint[i];
				mob[i].mobStyle=map[mapIndex].mobStyle[i];
				mob[i].vel.x=2.0f+(1-rand()%3);
				mob[i].isDead=false;
				mob[i].isDying=false;
				mob[i].isDropped=false;
				seconds[i].isActive=false;
				seconds[i].vel=sf::Vector2f(0,0);

			}
			if(mapIndex==5)
			{
				curState=gs_WIN;
				themeSong.Stop();
				themeSong.SetBuffer(clockSaver);
				themeSong.Play();
			}
			player.isAscended=false;
		}
	}

	app.Close();

}

bool GameClass::isState(GameStateEnum test)
{
	return test==curState;
}