//map.cpp

#include "globals.h"

MapClass::MapClass()
{
	startPoint=sf::Vector2f(0,0);
}

sf::IntRect MapClass::getBoundingTile(int index)
{
	return sf::IntRect(int(tile[index].pos.x), int(tile[index].pos.y), int(tile[index].pos.x)+32, int(tile[index].pos.y)+32);
}

void MapClass::initMap(const char *mapfile)
{
	std::ifstream file(mapfile);
	int tid=0;
	int meta=0;
	int ix=0;
	for(int y=0; y<16; y++)
	{
		for(int x=0; x<9; x++)
		{
			file >> tid;
			setTile(ix, ((tid<34 || tid>37) && (tid<10 || tid>13) && !(tid==20 || tid==21 || tid==28 || tid==29)), false, false, tid);
			ix++;
		}
	}
	ix=0;
	int imc=0;
	for(int y=0; y<16; y++)
	{
		for(int x=0; x<9; x++)
		{
			file >> meta;
			switch(meta)
			{
			case 1://mob bumper
				tile[ix].isMobBumper=true;
				break;
			case 2://gravity tile
				tile[ix].isGravity=true;
				break;
			case 3://portal tile
				tile[ix].isPortal=true;
				break;
			case 4://tick mob
				mobPoint[imc]=tile[ix].pos;
				mobStyle[imc]=16;
				imc++;
				break;
			case 5://sandlion mob
				mobPoint[imc]=tile[ix].pos;
				mobStyle[imc]=17;
				imc++;
				break;
			case 6://mystery mob
				mobPoint[imc]=tile[ix].pos;
				mobStyle[imc]=18;
				imc++;
				break;
			}
			ix++;
		}
	}
}

void MapClass::setTile(int index, bool isS, bool isL, bool isG, int id)
{
	float px=32.0f*float(index%16);
	float py=32.0f*float(int(index/16));

	tile[index].isSolid=isS;
	tile[index].isLethal=isL;
	tile[index].isGravity=isG;
	tile[index].vel=sf::Vector2f(0,0);
	tile[index].pos.x=px;
	tile[index].pos.y=py;
	tile[index].tileID=id;
}

void MapClass::setTile(int xx, int yy, bool isS, bool isL, bool isG, int id)
{
	float px=32.0f*float(xx);
	float py=32.0f*float(yy);
	int index=(yy*16)+xx;

	tile[index].isSolid=isS;
	tile[index].isLethal=isL;
	tile[index].isGravity=isG;
	tile[index].vel=sf::Vector2f(0,0);
	tile[index].pos.x=px;
	tile[index].pos.y=py;
	tile[index].tileID=id;
}

void MapClass::setVelocity(int index, sf::Vector2f nv)
{
	tile[index].vel=nv;
}

void MapClass::setVelocity(int xx, int yy, sf::Vector2f nv)
{
	int index=(yy*16)+xx;
	tile[index].vel=nv;
}

void MapClass::setGravity(int index, bool val)
{
	tile[index].isGravity=val;
}

void MapClass::setGravity(int xx, int yy, bool val)
{
	int index=(yy*16)+xx;
	tile[index].isGravity=val;
}

void MapClass::render(sf::RenderWindow& win)
{
	sf::Image img;
	img.LoadFromFile("images/spritesheet.png");
	img.SetSmooth(false);
	sf::Sprite spr;
	spr.SetImage(img);
	sf::IntRect r(0,0,0,0);
	for(int i=TILES_ON_MAP-1; i>=0; i--)
	{
		if(tile[i].tileID != 10)
		{
			int xx=tile[i].tileID%8;
			int yy=int(tile[i].tileID/8);
			r=sf::IntRect(xx*32, yy*32, (xx+1)*32, (yy+1)*32);
			spr.SetSubRect(r);
			spr.SetPosition(tile[i].pos);
			win.Draw(spr);
		}
	}
}

void MapClass::update()
{
	for(int i=0; i<TILES_ON_MAP; i++)
	{
		if(abs(tile[i].vel.x)>0.0f || abs(tile[i].vel.y)>0.0f)
		{
			tile[i].pos+=tile[i].vel;
		}
		if(tile[i].isGravity)
		{
			tile[i].vel.y+=0.05f;

			if(tile[i].vel.y>1.7f) tile[i].vel.y=1.7f;

			tile[i].pos.y+=tile[i].vel.y;
			for(int j=0; j<TILES_ON_MAP; j++)
			{
				if(i!=j)
				{
					if(getBoundingTile(j).Intersects(getBoundingTile(i)) || !getBoundingTile(i).Intersects(sf::IntRect(0,0,32*16, 32*9)))
					{
						if(tile[j].isSolid)
						{
							if(tile[i].vel.y>0.0f)
							{
								tile[i].pos.y-=tile[i].vel.y;
								tile[i].vel.y=0.0f;
							}
						}
					}
				}
			}
		}
	}
}