//mob.cpp
#include "globals.h"

sf::IntRect MobClass::getBoundingBox()
{
	if(mobStyle==16)
	{	if(isDead)
			return sf::IntRect(int(pos.x)+9, int(pos.y)+0, int(pos.x)+32, int(pos.y)+17);
		else
			return sf::IntRect(int(pos.x)+9, int(pos.y)+16, int(pos.x)+32, int(pos.y)+32);
	}
	else if(mobStyle==17)
	{
		if(isDead)
			return sf::IntRect(int(pos.x)+9, int(pos.y)+0, int(pos.x)+32, int(pos.y)+23);
		else
			return sf::IntRect(int(pos.x)+9, int(pos.y)+8, int(pos.x)+32, int(pos.y)+32);
	}
	else return sf::IntRect(int(pos.x)+9, int(pos.y)+16, int(pos.x)+32, int(pos.y)+32);
}

MobClass::MobClass()
{
	pos=sf::Vector2f(0,0);
	vel=sf::Vector2f(0,0);
	sf::Image im;
	im.LoadFromFile("images/spritesheet.png");
	im.SetSmooth(false);
	loadFrame(im, 0);
	isFalling=true;
	isJumping=false;
	isDying=false;
	isDead=false;
	isDropped=false;
	dyingClock.Reset();
	maxjump=6;
	jumptime=0;
	mobStyle=16;
}

void MobClass::loadFrame(sf::Image& img, int index)
{
	//split index into x/y coords based on an 8x8 sheet of 32x32 tiles
	//starting from top left corner '0' -> (0,0)
	int xx=index%8;
	int yy=int(index/8);
	sf::IntRect r(xx*32, yy*32, (xx+1)*32, (yy+1)*32);
	frame.SetImage(img);
	frame.SetSubRect(r);
}

void MobClass::render(sf::RenderWindow& win, sf::Image& spritesheet)
{
	if(isAttacking && attackClock.GetElapsedTime()<0.45f) loadFrame(spritesheet, mobStyle);
	else if(isJumping) loadFrame(spritesheet, mobStyle);
	else loadFrame(spritesheet, mobStyle);
	frame.SetPosition(pos);
	frame.FlipX(isLeft);
	frame.FlipY(isDead);
	win.Draw(frame);
}

void MobClass::update(MapClass& m)
{
	if(!isDead)
	{
		if(vel.x>0.0f)
		{
			isLeft=false;
		//	if(vel.x<0.2f) vel.x=0.0f;
		//	else vel.x-=0.2f;
		}
		if(vel.x>1.6f) vel.x=1.6f;

		if(vel.x<0.0f)
		{
			isLeft=true;
		//	if(vel.x>-0.2f) vel.x=0.0f;
		//	else vel.x+=0.2f;
		}
		if(vel.x<-1.6f) vel.x=-1.6f;
		pos.x+=vel.x;
	}
	for(int i=0; i<TILES_ON_MAP; i++)
	{
		if(getBoundingBox().Intersects(m.getBoundingTile(i)) || !getBoundingBox().Intersects(sf::IntRect(16,16,(32*16)-16, (32*9)-16)))
		{
			if(m.tile[i].isSolid || m.tile[i].isMobBumper)
			{
				pos.x-=vel.x;
				vel.x=-vel.x;
				break;
			}
		}
	}

	if(isFalling || !isJumping)
	{
		vel.y+=1.5f;
	}
	else if(isJumping)
	{
		isFalling=true;
		jumptime++;
		if(jumptime>=maxjump)
		{
			isFalling=true;
		}
	}

	if(vel.y>5.7f) vel.y=5.7f;

	pos.y+=vel.y;
	for(int i=0; i<TILES_ON_MAP; i++)
	{
		if(getBoundingBox().Intersects(m.getBoundingTile(i)) || !getBoundingBox().Intersects(sf::IntRect(16,16,32*16, 32*9)))
		{
			if(m.tile[i].isSolid)
			{
				if(vel.y>0.0f)
				{
					pos.y-=vel.y;
					vel.y=0.0f;
					isJumping=false;
					isFalling=false;
					jumptime=0;
				}
				else if(vel.y<0.0f)
				{
					pos.y-=vel.y;
					vel.y=0.0f;
					isJumping=false;
					isFalling=false;
					jumptime=maxjump;
				}
			}
		}
	}

	if(isAttacking && attackClock.GetElapsedTime()>0.45f)
	{
		isAttacking=false;
	}
	if(isDying && dyingClock.GetElapsedTime()>0.45f)
	{
		isDying=false;
		isDead=true;
	}

	frame.SetPosition(pos);

}