//player.cpp
#include "globals.h"

sf::IntRect PlayerClass::getBoundingBox()
{
	return sf::IntRect(int(pos.x)+9, int(pos.y)+6, int(pos.x)+29, int(pos.y)+29);
}

sf::IntRect PlayerClass::getAttackBox()
{
	if(isLeft)
		return sf::IntRect(int(pos.x)-9, int(pos.y)+6, int(pos.x)+11, int(pos.y)+29);
	else
		return sf::IntRect(int(pos.x)+27, int(pos.y)+6, int(pos.x)+47, int(pos.y)+29);

}

PlayerClass::PlayerClass()
{
	init();
}

void PlayerClass::init()
{
	pos=sf::Vector2f(0,0);
	vel=sf::Vector2f(0,0);
	sf::Image im;
	im.LoadFromFile("images/spritesheet.png");
	im.SetSmooth(false);
	loadFrame(im, 0);
	isFalling=true;
	isJumping=false;
	isDead=false;
	isAscended=false;
	maxjump=6;
	jumptime=0;
	collected=0;
	kaching.LoadFromFile("sounds/bling.wav");
	coinSound.SetLoop(false);
}

void PlayerClass::loadFrame(sf::Image& img, int index)
{
	//split index into x/y coords based on an 8x8 sheet of 32x32 tiles
	//starting from top left corner '0' -> (0,0)
	int xx=index%8;
	int yy=int(index/8);
	sf::IntRect r(xx*32, yy*32, (xx+1)*32, (yy+1)*32);
	frame.SetImage(img);
	frame.SetSubRect(r);
}

void PlayerClass::render(sf::RenderWindow& win, sf::Image& spritesheet)
{
	if(isAttacking && attackClock.GetElapsedTime()<0.45f) loadFrame(spritesheet, 2);
	else if(isJumping) loadFrame(spritesheet, 1);
	else loadFrame(spritesheet, 0);
	frame.SetPosition(pos);
	frame.FlipX(isLeft);
	frame.FlipY(isDead);
	win.Draw(frame);
}

void PlayerClass::update(MapClass& m, MobClass* enemyPtr, SecondsClass* secPtr)
{
	if(vel.x>0.0f)
	{
		isLeft=false;
		if(vel.x<0.2f) vel.x=0.0f;
		else vel.x-=0.2f;
	}
	if(vel.x>3.6f) vel.x=3.6f;

	if(vel.x<0.0f)
	{
		isLeft=true;
		if(vel.x>-0.2f) vel.x=0.0f;
		else vel.x+=0.2f;
	}
	if(vel.x<-3.6f) vel.x=-3.6f;
	pos.x+=vel.x;

	if(isAttacking)
	{
		for(int i=0; i<10; i++)
		{
			if(enemyPtr[i].getBoundingBox().Intersects(getAttackBox()))
			{
				enemyPtr[i].isDying=true;
				enemyPtr[i].vel.y=-6.0f;
			}
		}
	}
	else
	{
		for(int i=0; i<10; i++)
		{
			if(!(enemyPtr[i].isDying || enemyPtr[i].isDead) && enemyPtr[i].getBoundingBox().Intersects(getBoundingBox()))
			{
				int rn=10-(rand()%20);
				vel=sf::Vector2f(float(rn*2), -16.0f);
				isDead=true;
				deathClock.Reset();
			}
			if(secPtr[i].isActive && secPtr[i].getBoundingBox().Intersects(getBoundingBox()))
			{
				collected++;
				coinSound.Play();
				secPtr[i].isActive=false;
			}
		}
	}
	for(int i=0; i<TILES_ON_MAP; i++)
	{
		if(m.getBoundingTile(i).Contains(int(pos.x+16), int(pos.y+16)) && m.tile[i].isSolid)
		{
			isDead=true;
		}
		if(m.getBoundingTile(i).Contains(int(pos.x+6), int(pos.y+16)) && m.tile[i].isPortal && !isDead && collected>=10)
		{
			isAscended=true;
		}
		if(getBoundingBox().Intersects(m.getBoundingTile(i)) || !getBoundingBox().Intersects(sf::IntRect(16,16,(32*16)-16, (32*9)-16)))
		{
			if(m.tile[i].isSolid)
			{
				pos.x-=vel.x;
				vel.x=0.0f;
				break;
			}
		}
	}

	if(isFalling || !isJumping)
	{
		vel.y+=1.5f;
	}
	else if(isJumping)
	{
		isFalling=true;
		jumptime++;
		if(jumptime>=maxjump)
		{
			isFalling=true;
		}
	}

	if(vel.y>5.7f) vel.y=5.7f;

	pos.y+=vel.y;
	for(int i=0; i<TILES_ON_MAP; i++)
	{
		if(getBoundingBox().Intersects(m.getBoundingTile(i)) || !getBoundingBox().Intersects(sf::IntRect(16,16,32*16, 32*9)))
		{
			if(m.tile[i].isSolid)
			{
				if(vel.y>0.0f)
				{
					pos.y-=vel.y;
					vel.y=0.0f;
					isJumping=false;
					isFalling=false;
					jumptime=0;
				}
				else if(vel.y<0.0f)
				{
					pos.y-=vel.y;
					vel.y=0.0f;
					isJumping=false;
					isFalling=false;
					jumptime=maxjump;
				}
			}
		}
	}

	if(isAttacking && attackClock.GetElapsedTime()>0.45f)
	{
		isAttacking=false;
	}

	frame.SetPosition(pos);

}